@@ -52,6 +52,7 @@
 
 G_BEGIN_DECLS
 
+#define VTE_CURSOR_OUTLINE		1
 #define VTE_PAD_WIDTH			1
 #define VTE_TAB_WIDTH			8
 #define VTE_LINE_WIDTH			1
@@ -95,27 +96,32 @@ G_BEGIN_DECLS
 struct vte_charcell {
 	gunichar c;		/* The Unicode character. */
 
-	guint32 columns: 2;	/* Number of visible columns (as determined
-				   by g_unicode_iswide(c)). */
-	guint32 fore: 9;	/* Indices in the color palette for the */
-	guint32 back: 9;	/* foreground and background of the cell. */
+	struct vte_charcell_attr {
+		guint32 columns: 4;	/* Number of visible columns
+					   (as determined by g_unicode_iswide(c)).
+					   Also abused for tabs; bug 353610
+					   Keep at least 4 for tabs to work
+					   */
+		guint32 fore: 9;	/* Index into color palette */
+		guint32 back: 9;	/* Index into color palette. */
 
-	guint32 fragment: 1;	/* The nth fragment of a wide character. */
-	guint32 standout: 1;	/* Single-bit attributes. */
-	guint32 underline: 1;
-	guint32 strikethrough: 1;
+		guint32 fragment: 1;	/* A continuation cell. */
+		guint32 standout: 1;	/* Single-bit attributes. */
+		guint32 underline: 1;
+		guint32 strikethrough: 1;
 
-	guint32 reverse: 1;
-	guint32 blink: 1;
-	guint32 half: 1;
-	guint32 bold: 1;
+		guint32 reverse: 1;
+		guint32 blink: 1;
+		guint32 half: 1;
+		guint32 bold: 1;
 
-	guint32 invisible: 1;
-	guint32 protect: 1;
-	guint32 alternate: 1;
-	guint32 empty : 1;
+		guint32 invisible: 1;
+		/* unused; bug 499893
+		guint32 protect: 1;
+		 */
 
-	/* no more bits left.  any addition will enlarge the struct */
+		/* 31 bits */
+	} attr;
 };
 
 /* A match regex, with a tag. */
@@ -236,6 +242,7 @@ struct _VteTerminalPrivate {
 							   fore/back with no
 							   character data */
 		struct vte_charcell basic_defaults;	/* original defaults */
+		gboolean alternate_charset;
 		gboolean status_line;
 		GString *status_line_contents;
 		gboolean status_line_changed;
@@ -275,8 +282,8 @@ struct _VteTerminalPrivate {
 	gboolean smooth_scroll;
 	GHashTable *tabstops;
 	gboolean text_modified_flag;
-	glong text_inserted_count;
-	glong text_deleted_count;
+	gboolean text_inserted_flag;
+	gboolean text_deleted_flag;
 
 	/* Scrolling options. */
 	gboolean scroll_background;
@@ -376,7 +383,11 @@ struct _VteTerminalPrivate {
 	/* Obscured? state. */
 	GdkVisibilityState visibility_state;
 
+	/* Font stuff. */
 	gboolean has_fonts;
+	glong line_thickness;
+	glong underline_position;
+	glong strikethrough_position;
 };
 
 
@@ -391,15 +402,14 @@ void _vte_invalidate_cursor_once(VteTerminal *terminal, gboolean periodic);
 VteRowData * _vte_new_row_data(VteTerminal *terminal);
 VteRowData * _vte_new_row_data_sized(VteTerminal *terminal, gboolean fill);
 VteRowData * _vte_reset_row_data (VteTerminal *terminal, VteRowData *row, gboolean fill);
+void _vte_free_row_data(VteRowData *row);
 void _vte_terminal_adjust_adjustments(VteTerminal *terminal);
 void _vte_terminal_queue_contents_changed(VteTerminal *terminal);
 void _vte_terminal_emit_text_deleted(VteTerminal *terminal);
 void _vte_terminal_emit_text_inserted(VteTerminal *terminal);
-void _vte_terminal_insert_char(VteTerminal *terminal, gunichar c,
+gboolean _vte_terminal_insert_char(VteTerminal *terminal, gunichar c,
 			       gboolean force_insert_mode,
-			       gboolean invalidate_cells,
-			       gboolean paint_cells,
-			       gint forced_width);
+			       gboolean invalidate_cells);
 void _vte_terminal_scroll_region(VteTerminal *terminal,
 				 long row, glong count, glong delta);
 void _vte_terminal_set_default_attributes(VteTerminal *terminal);
